/*#
#
# Mini Gtk-file-manager Copyright (C) 2002-2006 
# by Tito Ragusa <tito-wolit@tiscali.it>
#
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Please read the COPYING and README file!!!
#
# Report bugs to <tito-wolit@tiscali.it>
#
#
*/

#include <gtk/gtk.h>
#include <stdio.h>
#include <dirent.h>
#include <grp.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <unistd.h>
#include <time.h>
#include <libgen.h>
#include <string.h>
#include <errno.h>
#include <assert.h>
#include "mygtk.h"

#ifdef NO_CORE_DUMPS
	#include <sys/prctl.h>
#endif

static char *clist_selection = NULL;
static GtkWidget *window;
static GtkWidget *dirlist;
static GtkWidget *filelist;
static GtkWidget *info_label;
static GtkWidget *path_label;
static GtkWidget *gui_prep_dialog;
static struct stat stat_p;
static int action_num = 0;
static int show_hidden_files = FALSE;

static void set_info_label_centered(GtkWidget *info_text, char *info)
{
	shrink_widget(GTK_WIDGET(info_text), 0, -1);
	gtk_label_set_text (GTK_LABEL(info_text), g_locale_to_utf8(info, -1, NULL, NULL, NULL));
	gtk_misc_set_alignment((GtkMisc *)info_text,  0.5, 0.5);
}

static void dinfo(unsigned long dirlistmax, unsigned long filelistmax, unsigned long totalsize)
{
	char *info;

	DBG_FUNCTION_NAME;

	info = xmake_message("%lu %s     %lu %s     %s",
						dirlistmax - 2,
						((dirlistmax - 2 ) == 1) ? STR_DIRECTORY : STR_DIRECTORIES,
						filelistmax,
						((filelistmax) == 1) ? STR_FILE : STR_FILES,
						make_human_readable_str(totalsize, 1, 0));
	set_info_label_centered(info_label, info);
	xfree(&info);
}

static void list_f(void)
{
	int i, n;
	char *cwd = NULL;
	char *text[1] = {NULL};
	unsigned long dirlistmax = 0;
	unsigned long filelistmax = 0;
	unsigned long totalsize = 0;
	struct dirent  **namelist;

	DBG_FUNCTION_NAME;

	if ((cwd =  xgetcwd(cwd)) == NULL) {
		dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_ABORTING": "STR_CANNOT_GET_CWD_S, strerror(errno));
		return;
	}

	if ((n = scandir(cwd, &namelist, 0, alphasort)) < 0) {
		dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_CANNOT_SCAN_DIR_S_S, cwd, strerror(errno));
	} else {
		dirlistmax = filelistmax = totalsize = 0;
		for( i = 0; i < n && GTK_IS_WIDGET(window); i++ ) {
			if (file_exists(namelist[i]->d_name, &stat_p) == -1)
				continue;
			if (show_hidden_files == FALSE && namelist[i]->d_name[0] == '.'
										&& strcmp(namelist[i]->d_name, ".") != 0 
										&& strcmp(namelist[i]->d_name, "..") != 0)
				continue; 
			totalsize += stat_p.st_size;
			if (S_ISDIR(stat_p.st_mode)) {
				dirlistmax++;
				text[0] = xconcat_path_file(namelist[i]->d_name, "");
				add_data_to_list(dirlist, text, 1, NO_AUTOSCROLL);
				xfree(&text[0]);
			} else {
				filelistmax++;
				text[0] = namelist[i]->d_name;
				add_data_to_list(filelist, text, 1, NO_AUTOSCROLL);
			}
			xfree(&namelist[i]);
		}
		xfree(&namelist);
		gtk_label_set_text (GTK_LABEL(path_label), cwd);
		xfree(&cwd);
		dinfo(dirlistmax, filelistmax, totalsize);
	}
	/* Reset the previous selection to NULL */ 
	clist_selection = NULL;
}

static void update(void)
{
	DBG_FUNCTION_NAME;

	clear_list(dirlist);
	clear_list(filelist);
	list_f();
}

static int no_file_selected(char *selection)
{
	DBG_FUNCTION_NAME;
		
	if (selection) return 0;

 	dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_NO_FILE_SELECTED);
	return 1;
}

static  void run(char *command, char *arg, int flag)
{
	char * p_argv[3];

	DBG_FUNCTION_NAME;

	if (flag && no_file_selected(arg)) return;	

	p_argv[0] = command;
	p_argv[1] = arg;
	p_argv[2] = NULL;

	g_spawn_async(NULL, p_argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, NULL);
}

static  void view_text_win(void)
{
	run(VIEWER, clist_selection, 1);
}

static void view_image(void)
{
	run(IMAGE_VIEWER, clist_selection, 1);
}

static void run_shell (void)
{
	run(TERMINAL, NULL, 0);
}

static void edit_text_win (void)
{
	run(EDITOR, clist_selection, 0);
}

static void do_file_selection(GtkWidget *widget, GtkWidget *entry_box)
{
	char * path = NULL;
	char *tmp;

	DBG_FUNCTION_NAME;
	assert(widget != NULL);
	
	gtk_window_set_modal(GTK_WINDOW(gui_prep_dialog), FALSE);

	path = default_old_file_selector(STR_SELECT_PATH);

	if (path) {
		tmp = g_locale_to_utf8(path, -1,  NULL, NULL, NULL);
		gtk_entry_set_text(GTK_ENTRY(entry_box), tmp);
		xfree(&tmp);
		xfree(&path);
	}
	/*else  entry_box unchanged */
	gtk_window_set_modal(GTK_WINDOW(gui_prep_dialog), TRUE);
}

static void file_op (GtkWidget *widget, GtkWidget *entry_box)
{
	char *destname = NULL;
	char *new_destname = NULL;
	char *message = NULL;
	int status = 0;
	int flag;

	debug_err_msg("%s(): action_num %d",__FUNCTION__, action_num);

	if (action_num != ACTION_DELETE) {
		assert(widget != NULL);
		/* FIXME utf8 to locale */
		destname = xstrdup(gtk_entry_get_text (GTK_ENTRY (entry_box)));
		if (xstrlen(destname) == 0) {
			dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_MISSING_ARG);
			/* Reset action_num */
			action_num = ACTION_NULL;
			goto END;
		}
	}
	if (action_num != ACTION_MKDIR) {
		if (no_file_selected(clist_selection)) {
			/* Reset action_num */
			action_num = ACTION_NULL;
			goto END;
		}
	}
		
	switch(action_num) {
		case ACTION_COPY:
			/*Remove |FILEUTILS_INTERACTIVE if you don't want to be asked
			for confirmation to overwrite for every single file */
			if (is_directory(destname, TRUE, NULL) == TRUE && is_directory(clist_selection , TRUE, NULL) == FALSE) {
				new_destname = xconcat_path_file( destname , clist_selection);
				status = copy_file(clist_selection,new_destname, 
									FILEUTILS_RECUR|FILEUTILS_PRESERVE_STATUS|FILEUTILS_INTERACTIVE);
				xfree(&new_destname);
			} else {
				status = copy_file(clist_selection, destname, 
									FILEUTILS_RECUR|FILEUTILS_PRESERVE_STATUS|FILEUTILS_INTERACTIVE);
			}
			message = STR_FAILED_TO_COPY_S_TO_S_S;
			break;
		case ACTION_HARDLINK:
			status = link(clist_selection, destname);
			message = STR_FAILED_TO_CREATE_HARD_LINK_TO_S_WITH_NAME_S_S;
			break;
		case ACTION_SYMLINK:
			status = symlink(clist_selection, destname);
			message = STR_FAILED_TO_CREATE_SYM_LINK_TO_S_WITH_NAME_S_S;
			break;
		case ACTION_MOVE:
			status = move_rename(clist_selection, destname, OPT_FILEUTILS_INTERACTIVE);
			message = STR_CANNOT_MOVE_RENAME_S_TO_S_S;
			break;
		case ACTION_MKDIR:
			if (file_exists(destname, NULL) == 0) {
				message = STR_FAILED_TO_CREATE_DIRECTORY_S":\n"STR_FILE_EXISTS;
				status =-1;
			} else {
				status = bb_make_directory (destname, 0755, FILEUTILS_RECUR| FILEUTILS_INTERACTIVE);
				message = STR_FAILED_TO_CREATE_DIRECTORY_S;
			}
			/* for the dialog window message */
			clist_selection = destname;
			break;
		case ACTION_DELETE:
			/*Add |FILEUTILS_INTERACTIVE if you want to be asked
			for confirmation to delete for every single file */
			flag = FILEUTILS_RECUR /*| FILEUTILS_INTERACTIVE*/;
			if (!(flag & FILEUTILS_INTERACTIVE)) {
				/* if we use  FILEUTILS_INTERACTIVE we don't need this gui */
				if (dialog_window(STR_DELETE, TWO_BUTTONS, NULL, 2, STR_REALLY_DELETE_S, clist_selection ) != 0) {
					break;
				}
			}
			status = remove_file(basename(clist_selection) , flag);
			message = STR_FAILED_TO_DELETE_S;
			/* for the dialog window message */
			destname = clist_selection;
			break;
		default:
			debug_err_msg("Unknown ACTION");
			goto END;
	}
	if (status) {
		dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, message , clist_selection, destname, strerror(errno));
	}
END:
	if (action_num != ACTION_DELETE) {
		window_close(widget, gui_prep_dialog); 
		xfree(&destname);
	}
	/* Reset action_num */
	action_num = ACTION_NULL;
	update();
}

static void flag_link_action_callback(GtkWidget *widget, gpointer data)
{
	assert(widget != NULL);

	/* 2 = ACTION_SYMLINK  */
	/* 3 = ACTION_HARDLINK */
	action_num = atoi(data);
}

static void gui_prepare(char *title, char *label_text, char *button_label)
{
	GtkWidget *table;
	GtkWidget *label;
	GtkWidget *entry_box = NULL;
	GtkWidget *button;
	GtkWidget *radio_button;
	GSList    *group;
	
	DBG_FUNCTION_NAME;
		
	gui_prep_dialog = window_create(380,180,2, title, GTK_SIGNAL_FUNC(gtk_main_quit), NULL, MODAL);
	set_position (GTK_WINDOW (gui_prep_dialog), GTK_WIN_POS_MOUSE);
	table = table_create(12,5, gui_prep_dialog, 2, HOMOGENEUS);
	label = default_label_create_on_table(label_text, table, 1,11,0,1);
	if (action_num != ACTION_DELETE) {
		entry_box = default_text_box_create_on_table("", EDITABLE, table, 1,11,1,2);
		gtk_widget_grab_focus (entry_box);
		/* Create a custom button for the file selector */
		button = button_create_on_table(":",GTK_SIGNAL_FUNC(do_file_selection), entry_box, table, 11, 12, 1, 2);
		shrink_widget(button , -1, -1);
	}

	if (action_num == ACTION_SYMLINK) {
		radio_button = radio_create_on_table(NULL,	STR_SYMBOLIC_LINK,
											GTK_SIGNAL_FUNC(flag_link_action_callback),
											"2", ACTIVE,table, 0, 6, 3, 4);
		group = get_radio_button_group (GTK_RADIO_BUTTON (radio_button));
		radio_button = radio_create_on_table(radio_button,	STR_HARD_LINK,
											GTK_SIGNAL_FUNC(flag_link_action_callback),
											"3", INACTIVE,table, 0, 6, 4, 5);
	}
	
	button = button_create_on_table(STR_QUIT, GTK_SIGNAL_FUNC(window_close), gui_prep_dialog, table, 6, 8, 4, 5);
	gtk_widget_grab_focus(button);
	
	if (action_num == ACTION_MOVE) {
		button = button_create_on_table( button_label, GTK_SIGNAL_FUNC(file_op), entry_box, table, 8 , 12, 4, 5);
		shrink_widget(button, 1, -1);
	} else {
		button = button_create_on_table(button_label, GTK_SIGNAL_FUNC(file_op), entry_box, table, 9, 12, 4, 5);
	}
}

static int is_dot_or_double_dot(char *selection)
{
	DBG_FUNCTION_NAME;
	
	if (no_file_selected(selection))
		return 1;

	if (strcmp(selection,".") == 0 
		|| strcmp(selection,"..") == 0 
		|| strcmp(selection,"./") == 0
		|| strcmp(selection,"../") == 0) {
		dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_CANNOT_OPERATE_ON_DOT_OR_DOUBLE_DOT);
		return 1;
	}
	return 0;
}

static void gui_common(char* title, char *fmt)
{
	char *message = NULL;

	DBG_FUNCTION_NAME;
	
	if (is_dot_or_double_dot(clist_selection)) return;
		
	message = xmake_message(fmt, clist_selection);
	gui_prepare(title, message, title);
	main_loop(&gui_prep_dialog);
	xfree(&message);
}

static void move_gui(void)
{
	DBG_FUNCTION_NAME;
	
	action_num = ACTION_MOVE;
	gui_common(STR_MOVE_RENAME, STR_MOVE_RENAME_S_TO);
}

static void create_dir_gui(void)
{
	action_num = ACTION_MKDIR;
	gui_prepare(STR_CREATE_A_NEW_DIR, STR_ENTER_DIR_NAME, STR_CREATE);
	main_loop(&gui_prep_dialog);
}

void delete_no_gui(void)
{
	if (is_dot_or_double_dot(clist_selection)) return;

	action_num = ACTION_DELETE;
	file_op(NULL, NULL);
}

static void copy_gui(void)
{
	action_num = ACTION_COPY;
	gui_common(STR_COPY, STR_COPY_S_TO);
}

static void link_gui(void)
{
	action_num = ACTION_SYMLINK;
	gui_common(STR_LINK, STR_CREATE_A_LINK_TO_S_WITH_NAME);
}

static void perm_gui(void)
{
	if (is_dot_or_double_dot(clist_selection)) return;
	 
	if (PermissionWindow(clist_selection)) update();
}

static const char *format_time(time_t cal_time)
{
	struct tm *time_struct;
	static char string[35];

	time_struct = localtime(&cal_time);
	strftime(string, 35, "%d %b %Y %H:%M", time_struct);
	return string;
}

static void get_info(char *filename)
{
	char *info = NULL;
	char *uid = NULL;
	char *gid = NULL;
	const char *m_time = NULL;
	const char *mode = NULL;
	char *link_path = NULL;
	char *link_msg = NULL;
	unsigned long filesize;

	debug_err_msg("%s(): %s",__FUNCTION__, filename);

	if (file_exists(filename, &stat_p) == 0) {
		uid = get_uid_or_gid(UID,stat_p.st_uid);
		gid = get_uid_or_gid(GID,stat_p.st_gid);
		m_time = format_time(stat_p.st_mtime);
		mode = bb_mode_string(stat_p.st_mode);
		filesize = stat_p.st_size;
		link_path = xreadlink(basename(filename));

		if (link_path) {
			link_msg =  (stat(link_path,&stat_p)) ?  STR_IS_BROKEN_LINK : STR_IS_LINK;
		}

		info = xmake_message("%s  %s %s  %s%s  %s  %s%s",
					mode, uid, gid,
					make_human_readable_str(filesize, 1, 0),
					(filesize <= 1024) ? "b" : "",
					m_time,
					(link_path) ? link_msg   : "",
					(link_path) ? link_path  : "" );

		set_info_label_centered(info_label, info);

		/* m_time, mode, uid, gid are static buffers, so don't free them */
		xfree(&info);
		xfree(&link_path);
	} else {
		/* maybe someone else changed the file's name or deleted it ? */
		update();
	}
}

static void flist_f(GtkWidget *tree)
{
	GtkTreeIter iter;
	GtkTreeModel *model;
	GtkTreeSelection *selection;
	char *tmp;

	if (GTK_IS_TREE_VIEW(tree))
		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
	else
		selection = GTK_TREE_SELECTION(tree);

	if (gtk_tree_selection_get_selected ( selection, &model, &iter)) {
		gtk_tree_model_get (model, &iter, FILE_COLUMN , &tmp, -1);
		clist_selection = g_locale_from_utf8(tmp, -1,  NULL, NULL, NULL);
		xfree(&tmp);
		debug_err_msg("%s(): you selected : %s",__FUNCTION__, clist_selection);
		gtk_tree_selection_unselect_all (gtk_tree_view_get_selection (GTK_TREE_VIEW (dirlist)));
		gtk_widget_show(dirlist);
		get_info(clist_selection);
	}
}

static void dirlist_select( GtkWidget *tree)
{
	GtkTreeIter iter;
	GtkTreeModel *model;
	GtkTreeSelection *selection;
	char *tmp;
	
	if (GTK_IS_TREE_VIEW(tree))
		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
	else
		selection = GTK_TREE_SELECTION(tree);

	if (gtk_tree_selection_get_selected ( selection, &model, &iter)) {
		gtk_tree_model_get (model, &iter, FILE_COLUMN , &tmp, -1);
		clist_selection = g_locale_from_utf8(tmp, -1, NULL, NULL, NULL);
		xfree(&tmp);
		debug_err_msg("%s(): you selected : %s",__FUNCTION__, clist_selection);
		gtk_tree_selection_unselect_all (gtk_tree_view_get_selection (GTK_TREE_VIEW (filelist)));
		gtk_widget_show(filelist);
		get_info(clist_selection);
	}
}

static void dirlist_f(void)
{
	if (xchdir(clist_selection, RETURN_ON_FAILURE)) {
		dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, 
						STR_CANNOT_CD_TO_DIR_S_S, clist_selection, strerror(errno));
	}
	update();
}

static void really_quit(GtkWidget *calling_widget)
{
	DBG_FUNCTION_NAME;

	if (dialog_window(VERSION, TWO_BUTTONS, NULL, 2, STR_REALLY_QUIT) == 0) {
		window_close(calling_widget, window);
		exit(EXIT_SUCCESS);
	}
}

static void toggle_hidden_files (void)
{
	show_hidden_files = !show_hidden_files;
	update();
}

static GtkActionEntry fm_menu_items[] = {
	/* name,		stock id,		label,			accelerator,		tooltip,	callback */
	{ "FileMenu",	NULL,		MNU_STR_FILE,				NULL,				NULL,		NULL},
	{ "ToolsMenu",	NULL,		MNU_STR_TOOLS,				NULL,				NULL,		NULL},	
	{ "OptionsMenu",NULL,		MNU_STR_OPTIONS,			NULL,				NULL,		NULL},	
	{ "HelpMenu",	NULL,		MNU_STR_HELP,				NULL,				NULL,		NULL},
	{ "Link",		NULL,		MNU_STR_LINK, 				"F9",				NULL,		G_CALLBACK (link_gui) },   
	{ "Delete",		NULL,		MNU_STR_DELETE,				"F8",				NULL,		G_CALLBACK (delete_no_gui) }, 
	{ "Mkdir",		NULL,		MNU_STR_MKDIR,				"F7",				NULL,		G_CALLBACK (create_dir_gui) },
	{ "Move",		NULL,		MNU_STR_MOVE,				"F6",				NULL,		G_CALLBACK (move_gui) },
	{ "Copy",		NULL,		MNU_STR_COPY,				"F5",				NULL,		G_CALLBACK (copy_gui) },
	{ "Edit",		NULL,		MNU_STR_EDIT,				"F4",				NULL,		G_CALLBACK (edit_text_win) },
	{ "View",		NULL,		MNU_STR_VIEW,				"F3",				NULL,		G_CALLBACK (view_text_win) },
	{ "Shell",		NULL,		MNU_STR_SHELL,				"F2",				NULL,		G_CALLBACK (run_shell) },
	{ "Perm",		NULL,		MNU_STR_PERM,				"F1",				NULL,		G_CALLBACK (perm_gui) },	
	{ "Quit",		NULL,		MNU_STR_QUIT,				SHC_FILE_QUIT,		NULL,		G_CALLBACK (really_quit) },
	{ "Refresh",	NULL,		MNU_STR_REFRESH,			SHC_REFRESH,		NULL,		G_CALLBACK (update) },
	{ "FSInfo",		NULL,		MNU_STR_FS_INFO,			SHC_FS_INFO,		NULL,		G_CALLBACK (df_window) },	
	{ "PSInfo",		NULL,		MNU_STR_PROCESS_MANAGER,	SHC_PROCESS_MANAGER,NULL,		G_CALLBACK (ps_window) },	
	{ "ViewImage",	NULL,		MNU_STR_VIEW_IMAGE,			SHC_VIEW_IMAGE,		NULL,		G_CALLBACK (view_image) },	
	{ "About",		NULL,		MNU_STR_ABOUT,				SHC_HELP_ABOUT,		NULL,		G_CALLBACK (about_cb) },
};

static GtkToggleActionEntry HiddenFiles_check_entries[] = {
{	
	"HiddenFiles", NULL,								/* name, stock id */
	MNU_STR_SHOW_HIDDEN_FILES, SHC_SHOW_HIDDEN_FILES,	/* label, accelerator */
	NULL,												/* tooltip */
	G_CALLBACK (toggle_hidden_files),					/* callback */ 
	FALSE },											/* is_active */
};

static const gchar *fm_ui_info = 
"<ui>"
"  <menubar name='MenuBar'>"
"    <menu action='FileMenu'>"
"      <menuitem action='Link'/>"
"      <menuitem action='Delete'/>"
"      <menuitem action='Mkdir'/>"
"      <menuitem action='Move'/>"
"      <menuitem action='Copy'/>"
"      <menuitem action='Edit'/>"
"      <menuitem action='View'/>"
"      <menuitem action='Shell'/>"
"      <menuitem action='Perm'/>"
"      <separator/>"
"      <menuitem action='Quit'/>"
"    </menu>"
"    <menu action='ToolsMenu'>"
"      <menuitem action='Refresh'/>"
"      <menuitem action='ViewImage'/>"
"      <menuitem action='FSInfo'/>"
"      <menuitem action='PSInfo'/>"
"    </menu>"
"    <menu action='OptionsMenu'>"
"      <menuitem action='HiddenFiles'/>"
"    </menu>"
"    <menu action='HelpMenu'>"
"      <menuitem action='About'/>"
"    </menu>"
"  </menubar>"
"</ui>";

static struct MenuEntry fm_menu_entries[] = {
{ fm_menu_items,		NULL,						NULL,			G_N_ELEMENTS (fm_menu_items)	,			0, NULL},
{ NULL,					HiddenFiles_check_entries,	NULL,			G_N_ELEMENTS (HiddenFiles_check_entries),	0, NULL},
{ NULL,					NULL,						NULL,			0,											0, NULL},
/* Set last to NULL */ 
};

int main (int argc, char **argv)
{
	GtkWidget *table;
	GtkWidget *statusbar;
	GtkWidget *button;
	GtkWidget *menubar;
	char *progname = basename(argv[0]);

	debug_err_msg("Program called as: %s",progname);

	/* print disclaimer to terminal */
	printf("\n"VERSION"\n\n"COPYRIGHT"\n"STR_LICENSE"\n"HOMEPAGE"\n"STR_WARRANTY"\n\n\n");

	/* gtk_init() automatically initializes internationalization support */ 
	gtk_init (&argc, &argv);

	/* Force the output to be unbuffered when debugging.*/
	debug(setbuf(stdout, (char *) 0));
	
	#ifdef NO_CORE_DUMPS
	/* make the process not dumpable under Linux 2.4, seems to work also for 2.6.0-test3 */
	prctl(PR_SET_DUMPABLE,0);
	debug_err_msg("process is%sdumpable",(prctl(PR_GET_DUMPABLE) == 0) ? " not " : " ");
	#endif

	#ifndef USE_EXTERNAL_EDITOR	
	if (strcmp(progname, basename(EDITOR)) == 0) {
		/* Run the built in text editor */
		textedit_window_create((argc > 1) ? argv[1] : "", 530, 450, 8, (argc > 1) ? argv[1] : NULL);
	} else
	#endif
	#ifndef USE_EXTERNAL_TEXT_VIEWER 
	if (strcmp(progname, basename(VIEWER)) == 0) {
		/* Run the built in text viewer */
		textview_window_create((argc > 1) ? argv[1] : NULL, 530, 450, 8, (argc > 1) ? argv[1] : NULL, NULL);
	} else
	#endif
	#ifndef USE_EXTERNAL_IMAGE_VIEWER	 
	if (strcmp(progname, basename(IMAGE_VIEWER)) == 0) {
		/* Run the built in image viewer */
		ViewImageWindow((argc > 1) ? argv[1] : "");
	} else
	#endif
	{
		/* Run the file manager */
		window = window_create(640, 450, 8, VERSION , GTK_SIGNAL_FUNC(really_quit), NULL, NOT_MODAL);
		table  = table_create(30, 30 , window , 5, HOMOGENEUS);
		menubar = main_menu_create(GTK_WIDGET (window), fm_menu_entries, fm_ui_info, NULL);
		gtk_table_attach_defaults (GTK_TABLE(table),GTK_WIDGET(menubar), 0, 30, 0, 2);
		statusbar = statusbar_create_on_table(table,&path_label,NULL,0, 30, 2, 4);

		dirlist = string_list_create_on_table(1, NULL, table ,  0, 15, 4, 26, SHOW, NOT_EDITABLE, " "STR_DIRECTORIES, 0.0);
		signal_connect (GTK_TREE_SELECTION(gtk_tree_view_get_selection (GTK_TREE_VIEW (dirlist))), "changed",
							G_CALLBACK (dirlist_select), NULL );
		signal_connect (GTK_TREE_VIEW (dirlist), "row-activated", G_CALLBACK (dirlist_f), NULL);

		filelist = string_list_create_on_table(1, NULL, table , 16, 30, 4, 26, SHOW, NOT_EDITABLE, " "STR_FILES, 0.0);
		signal_connect(gtk_tree_view_get_selection (GTK_TREE_VIEW (filelist)), "changed", G_CALLBACK (flist_f), NULL);
		statusbar = statusbar_create_on_table(table, &info_label, NULL, 0, 30, 26, 28);
				
		/* F10 seems not to work correctly with gtk+2.2.4 libs so we use alt+q here instead */
		button = button_create_on_table(STR_SHC_FILE_QUIT STR_QUIT, GTK_SIGNAL_FUNC(really_quit), NULL, table, 27, 30, 28, 30 );
		button = button_create_on_table("F9 "STR_LINK,	GTK_SIGNAL_FUNC(link_gui),		NULL, table, 24, 27, 28, 30 );
		button = button_create_on_table("F8 "STR_DELETE,GTK_SIGNAL_FUNC(delete_no_gui),	NULL, table, 21, 24, 28, 30 );
		button = button_create_on_table("F7 "STR_MKDIR,	GTK_SIGNAL_FUNC(create_dir_gui),NULL, table, 18, 21, 28, 30 );
		button = button_create_on_table("F6 "STR_MOVE, 	GTK_SIGNAL_FUNC(move_gui),		NULL, table, 15, 18, 28, 30 );
		button = button_create_on_table("F5 "STR_COPY,	GTK_SIGNAL_FUNC(copy_gui),		NULL, table, 12, 15, 28, 30 );
		button = button_create_on_table("F4 "STR_EDIT,	GTK_SIGNAL_FUNC(edit_text_win),	NULL, table,  9, 12, 28, 30 );
		button = button_create_on_table("F3 "STR_VIEW,	GTK_SIGNAL_FUNC(view_text_win),	NULL, table,  6,  9, 28, 30 );
		button = button_create_on_table("F2 "STR_SHELL,	GTK_SIGNAL_FUNC(run_shell),		NULL, table,  3,  6, 28, 30 );
		button = button_create_on_table("F1 "STR_PERM,	GTK_SIGNAL_FUNC(perm_gui),		NULL, table,  0,  3, 28, 30 );
		
		/* Don't remove this  */
		gtk_widget_show_all(window);
		/* Populate the lists */

		list_f();

		main_loop(&window);
	}
	exit(EXIT_SUCCESS);
}


